import traceback

import pyodbc

from wellog import loggen

from .Error import Error_Sql
from .Objects import Result

class SQLServer:
  def __init__(self):
    pass

  def select(self, key:str, query:str, params:list=None):
    status = True
    msg_error = ''

    data:dict = {}
    try:      
      conexion = pyodbc.connect(key)
      cursor = conexion.cursor()
      cursor.execute(query)
      columnas = [columna[0] for columna in cursor.description]
      data = cursor.fetchall()
      tabla = []
      for fila in data:
        fila_dict = {}
        for i in range(len(columnas)):
          fila_dict[columnas[i]] = fila[i]
        tabla.append(fila_dict)
      result = Result(data=tabla)
    except Exception as e:
      loggen.error(traceback.format_exc())
      status = False 
      msg_error = str(e)
    finally:
      cursor.close()
      conexion.close()
      if not status: raise Error_Sql(f'Error DB: {msg_error}', 501)
      return result


class Result:
  def __init__(
      self, 
      row_count:int = None,
      last_id:int = None,
      data:list = None
      ):
    self.row_count=row_count
    self.last_id=last_id
    self.data = data


import traceback

import mysql.connector

from wellog import loggen

from .Error import Error_Sql
from .Objects import Result

class MySQl:
  def __init__(self):
    pass

  def select(self, key: dict, query:str, params:list = None):
    status = True
    msg_error = ''

    data:dict = {}

    try:
      conexion = mysql.connector.connect(**key)
      cursor = conexion.cursor()    
      data['operation']=query
      if params:
        data['params']=params

      cursor.execute(**data)
      columnas = [columna[0] for columna in cursor.description]
      data = cursor.fetchall()
      tabla = []
      for fila in data:
        fila_dict = {}
        for i in range(len(columnas)):
          fila_dict[columnas[i]] = fila[i]
        tabla.append(fila_dict)
      result = Result(data=tabla)
    except Exception as e:
      loggen.error(traceback.format_exc())
      status = False 
      msg_error = str(e)
    finally:
      cursor.close()
      conexion.close()
      if not status: raise Error_Sql(f'Error DB: {msg_error}', 501)
      return result
  
  def execute(self, key: dict, query:str, params:list = None):
    status = True
    msg_error = ''

    data:dict = {}

    try:
      conexion = mysql.connector.connect(**key)
      cursor = conexion.cursor()
      data['operation']=query
      if params:
        data['params']=params

      cursor.execute(**data)
      conexion.commit()
      result = Result(
        row_count=cursor.rowcount,
        last_id=cursor.lastrowid
      )
    except Exception as e:
      loggen.error(traceback.format_exc())     
      status = False 
      msg_error = str(e)
    finally: 
      cursor.close()
      conexion.close()
      if not status: raise Error_Sql(f'Error DB: {msg_error}', 501)
      return result
    


class Error_Sql(Exception):
  '''Data Base'''
  def __init__(self, mensaje, code):
    self.mensaje = mensaje
    self._code = code

  def __str__(self):
        return self.mensaje
  
  @property
  def message(self):
     return self.mensaje
  
  @property
  def code(self):
     return self._code



from setuptools import find_packages, setup

with open('./README.md', 'r', encoding='utf-8') as fh:
  readme = fh.read()

setup(
  name='WdSql',
  version='0.0.3',
  description='Funciones basicas para bases de datos relacionales',
  long_description=readme,
  long_description_content_type='text/markdown',
  author='Pedro Jesús Pérez Hernández',
  author_email='pedrojesus.perez@welldex.mx',
  url='https://gitwell.gwldx.com:2443/python_libraries/wsql',
  install_requires=[
    'mysql-connector',
    'pyodbc',
    'wellog',
  ],
  license='MIT',
  packages=find_packages(),
  include_package_data=True,
  python_requires='>=3.8'
)